/*-------------------------------------------------------------------------
   Register Declarations for the Intel ADuC841 Processor

   Written By - Marek Piwonski <marek@microster.pl> (August 2009)

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

   In other words, you are welcome to use, share and improve this program.
   You are forbidden to forbid anyone else to use, share and improve
   what you give them.   Help stamp out software-hoarding!
-------------------------------------------------------------------------*/

#ifndef __ADUC841_H
#define __ADUC841_H

#include <asm/8052.h>     /* load difinitions for the 8052 core */

/*  BYTE Register  */
DEFINE_SFR(P0, 0x80)
DEFINE_SFR(SP, 0x81)
DEFINE_SFR(DPL, 0x82)
DEFINE_SFR(DPH, 0x83)
DEFINE_SFR(DPP, 0x84)
DEFINE_SFR(PCON, 0x87)
DEFINE_SFR(TCON, 0x88)
DEFINE_SFR(TMOD, 0x89)
DEFINE_SFR(TL0, 0x8A)
DEFINE_SFR(TL1, 0x8B)
DEFINE_SFR(TH0, 0x8C)
DEFINE_SFR(TH1, 0x8D)
DEFINE_SFR(P1, 0x90)
DEFINE_SFR(I2CADD1, 0x91)
DEFINE_SFR(I2CADD2, 0x92)
DEFINE_SFR(I2CADD3, 0x93)
DEFINE_SFR(SCON, 0x98)
DEFINE_SFR(SBUF, 0x99)
DEFINE_SFR(I2CDAT, 0x9A)
DEFINE_SFR(I2CADD, 0x9B)
DEFINE_SFR(T3FD, 0x9D)
DEFINE_SFR(T3CON, 0x9E)
DEFINE_SFR(P2, 0xA0)
DEFINE_SFR(TIMECON, 0xA1)
DEFINE_SFR(HTHSEC, 0xA2)
DEFINE_SFR(SEC, 0xA3)
DEFINE_SFR(MIN, 0xA4)
DEFINE_SFR(HOUR, 0xA5)
DEFINE_SFR(INTVAL, 0xA6)
DEFINE_SFR(DPCON, 0xA7)
DEFINE_SFR(IE, 0xA8)
DEFINE_SFR(IEIP2, 0xA9)
DEFINE_SFR(PWMCON, 0xAE)
DEFINE_SFR(CFG841, 0xAF)
DEFINE_SFR(P3, 0xB0)
DEFINE_SFR(PWM0L, 0xB1)
DEFINE_SFR(PWM0H, 0xB2)
DEFINE_SFR(PWM1L, 0xB3)
DEFINE_SFR(PWM1H, 0xB4)
DEFINE_SFR(SPH, 0xB7)
DEFINE_SFR(IP, 0xB8)
DEFINE_SFR(ECON, 0xB9)
DEFINE_SFR(EDATA1, 0xBC)
DEFINE_SFR(EDATA2, 0xBD)
DEFINE_SFR(EDATA3, 0xBE)
DEFINE_SFR(EDATA4, 0xBF)
DEFINE_SFR(WDCON, 0xC0)
DEFINE_SFR(CHIPID, 0xC2)
DEFINE_SFR(EADRL, 0xC6)
DEFINE_SFR(EADRH, 0xC7)
DEFINE_SFR(T2CON, 0xC8)
DEFINE_SFR(RCAP2L, 0xCA)
DEFINE_SFR(RCAP2H, 0xCB)
DEFINE_SFR(TL2, 0xCC)
DEFINE_SFR(TH2, 0xCD)
DEFINE_SFR(PSW, 0xD0)
DEFINE_SFR(DMAL, 0xD2)
DEFINE_SFR(DMAH, 0xD3)
DEFINE_SFR(DMAP, 0xD4)
DEFINE_SFR(PLLCON, 0xD7)
DEFINE_SFR(ADCCON2, 0xD8)
DEFINE_SFR(ADCDATAL, 0xD9)
DEFINE_SFR(ADCDATAH, 0xDA)
DEFINE_SFR(PSMCON, 0xDF)
DEFINE_SFR(ACC, 0xE0)
DEFINE_SFR(DCON, 0xE8)
DEFINE_SFR(I2CCON, 0xE8)
DEFINE_SFR(ADCCON1, 0xEF)
DEFINE_SFR(B, 0xF0)
DEFINE_SFR(ADCOFSL, 0xF1)
DEFINE_SFR(ADCOFSH, 0xF2)
DEFINE_SFR(ADCGAINL, 0xF3)
DEFINE_SFR(ADCGAINH, 0xF4)
DEFINE_SFR(ADCCON3, 0xF5)
DEFINE_SFR(SPIDAT, 0xF7)
DEFINE_SFR(SPICON, 0xF8)
DEFINE_SFR(DAC0L, 0xF9)
DEFINE_SFR(DAC0H, 0xFA)
DEFINE_SFR(DAC1L, 0xFB)
DEFINE_SFR(DAC1H, 0xFC)
DEFINE_SFR(DACCON, 0xFD)

/* BIT Register..... */
/* TCON */
DEFINE_SBIT(TF1, 0x8F)
DEFINE_SBIT(TR1, 0x8E)
DEFINE_SBIT(TF0, 0x8D)
DEFINE_SBIT(TR0, 0x8C)
DEFINE_SBIT(IE1, 0x8B)
DEFINE_SBIT(IT1, 0x8A)
DEFINE_SBIT(IE0, 0x89)
DEFINE_SBIT(IT0, 0x88)
/* P1 */
DEFINE_SBIT(T2EX, 0x91)
DEFINE_SBIT(T2, 0x90)
/* SCON */
DEFINE_SBIT(SM0, 0x9F)
DEFINE_SBIT(SM1, 0x9E)
DEFINE_SBIT(SM2, 0x9D)
DEFINE_SBIT(REN, 0x9C)
DEFINE_SBIT(TB8, 0x9B)
DEFINE_SBIT(RB8, 0x9A)
DEFINE_SBIT(TI, 0x99)
DEFINE_SBIT(RI, 0x98)
/* IE */
DEFINE_SBIT(EA, 0xAF)
DEFINE_SBIT(EADC, 0xAE)
DEFINE_SBIT(ET2, 0xAD)
DEFINE_SBIT(ES, 0xAC)
DEFINE_SBIT(ET1, 0xAB)
DEFINE_SBIT(EX1, 0xAA)
DEFINE_SBIT(ET0, 0xA9)
DEFINE_SBIT(EX0, 0xA8)
/* P3 */
DEFINE_SBIT(RD, 0xB7)
DEFINE_SBIT(WR, 0xB6)
DEFINE_SBIT(T1, 0xB5)
DEFINE_SBIT(T0, 0xB4)
DEFINE_SBIT(INT1, 0xB3)
DEFINE_SBIT(INT0, 0xB2)
DEFINE_SBIT(TXD, 0xB1)
DEFINE_SBIT(RXD, 0xB0)
/* IP */
DEFINE_SBIT(PSI, 0xBF)
DEFINE_SBIT(PADC, 0xBE)
DEFINE_SBIT(PT2, 0xBD)
DEFINE_SBIT(PS, 0xBC)
DEFINE_SBIT(PT1, 0xBB)
DEFINE_SBIT(PX1, 0xBA)
DEFINE_SBIT(PT0, 0xB9)
DEFINE_SBIT(PX0, 0xB8)
/* WDCON */
DEFINE_SBIT(PRE3, 0xC7)
DEFINE_SBIT(PRE2, 0xC6)
DEFINE_SBIT(PRE1, 0xC5)
DEFINE_SBIT(PRE0, 0xC4)
DEFINE_SBIT(WDIR, 0xC3)
DEFINE_SBIT(WDS, 0xC2)
DEFINE_SBIT(WDE, 0xC1)
DEFINE_SBIT(WDWR, 0xC0)
/* T2CON */
DEFINE_SBIT(TF2, 0xCF)
DEFINE_SBIT(EXF2, 0xCE)
DEFINE_SBIT(RCLK, 0xCD)
DEFINE_SBIT(TCLK, 0xCC)
DEFINE_SBIT(EXEN2, 0xCB)
DEFINE_SBIT(TR2, 0xCA)
DEFINE_SBIT(CNT2, 0xC9)
DEFINE_SBIT(CAP2, 0xC8)
/* PSW */
DEFINE_SBIT(CY, 0xD7)
DEFINE_SBIT(AC, 0xD6)
DEFINE_SBIT(F0, 0xD5)
DEFINE_SBIT(RS1, 0xD4)
DEFINE_SBIT(RS0, 0xD3)
DEFINE_SBIT(OV, 0xD2)
DEFINE_SBIT(F1, 0xD1)
DEFINE_SBIT(P, 0xD0)
/* ADCCON2 */
DEFINE_SBIT(ADCI, 0xDF)
DEFINE_SBIT(DMA, 0xDE)
DEFINE_SBIT(CCONV, 0xDD)
DEFINE_SBIT(SCONV, 0xDC)
DEFINE_SBIT(CS3, 0xDB)
DEFINE_SBIT(CS2, 0xDA)
DEFINE_SBIT(CS1, 0xD9)
DEFINE_SBIT(CS0, 0xD8)
/* I2CCON */
DEFINE_SBIT(MDO, 0xEF)
DEFINE_SBIT(I2CSI, 0xEF)
DEFINE_SBIT(MDE, 0xEE)
DEFINE_SBIT(I2CGC, 0xEE)
DEFINE_SBIT(MCO, 0xED)
DEFINE_SBIT(I2CID1, 0xED)
DEFINE_SBIT(MDI, 0xEC)
DEFINE_SBIT(I2CID0, 0xEC)
DEFINE_SBIT(I2CM, 0xEB)
DEFINE_SBIT(I2CRS, 0xEA)
DEFINE_SBIT(I2CTX, 0xE9)
DEFINE_SBIT(I2CI, 0xE8)
/* SPICON */
DEFINE_SBIT(ISPI, 0xFF)
DEFINE_SBIT(WCOL, 0xFE)
DEFINE_SBIT(SPE, 0xFD)
DEFINE_SBIT(SPIM, 0xFC)
DEFINE_SBIT(CPOL, 0xFB)
DEFINE_SBIT(CPHA, 0xFA)
DEFINE_SBIT(SPR1, 0xF9)
DEFINE_SBIT(SPR0, 0xF8)

/* CFG841 */
#define CFG841_XRAMEN	1
#define CFG841_PWPO	0x40
#define CFG841_DBUF	0x20

#define DPCON_AUTO	0x40	/* automatic switch */

#define DPCON_SP_NORMAL	0x00	/* shadow pointer normal */
#define DPCON_SP_INC	0x10	/* shadow pointer post-increment */
#define DPCON_SP_DEC	0x20	/* shadow pointer post-decrement */
#define DPCON_SP_SWAP	0x30	/* shadow pointer swap LSB */

#define DPCON_MP_NORMAL	0x00	/* shadow pointer normal */
#define DPCON_MP_INC	0x04	/* shadow pointer post-increment */
#define DPCON_MP_DEC	0x08	/* shadow pointer post-decrement */
#define DPCON_MP_SWAP	0x0c	/* shadow pointer swap LSB */
#define DPCON_DPSEL	0x01	/* select shadow pointer */

#define swap_dpconn()	do { DPCON++; } while (0)

#define PWMCON_SNGL		0x80	/* turn off P2.6/P3.4 */
#define PWMCON_MODE_DISABLE	0x00	/* PWM disabled */
#define PWMCON_MODE_SINGLE	0x10	/* Single on P2.7/P3.3 */
#define PWMCON_MODE_TWIN_8	0x20	/* two 8-bit PWM */
#define PWMCON_MODE_TWIN_16	0x30	/* two 16-bit PWM */
#define PWMCON_MODE_DUAL_NDAC	0x40	/* Dual NRZ 16-bit DAC*/
#define PWMCON_MODE_DUAL_PWM	0x50	/* Dual 8-bit PWM */
#define PWMCON_MODE_DUAL_DAC	0x60	/* Dual RZ 16-bit DAC */
#define PWMCON_MODE_MASK	0x70

#define PWMCON_CDIV_1		0x00	/* clock / 1 */
#define PWMCON_CDIV_4		0x04	/* clock / 4 */
#define PWMCON_CDIV_16		0x08	/* clock / 16 */
#define PWMCON_CDIV_64		0x0c	/* clock / 64 */
#define PWMCON_CDIV_MASK	0x0c

#define PWMCON_CSEL_PWM_15	0x00	/* XTAL / DIV / 15 */
#define PWMCON_CSEL_PWM		0x01	/* XTAL / DIV */
#define PWMCON_CSEL_EXT		0x02	/* external */
#define PWMCON_CSEL_XTAL	0x03	/* XTAL */
#define PWMCON_CSEL_MASK	0x03

#endif
