/*
 * tmos - interrupts.h
 *
 * Copyright (C) 2009  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __ASM_IRQS_H
#define __ASM_IRQS_H

#include <asm/system.h>
#include <asm/jiffies.h>

#if CONFIG_SPI_ADUC841_HIGHPRIO
#define ADUC841_SPI_BANK		HIGH_PRIORITY_BANK
#else
#define ADUC841_SPI_BANK		LOW_PRIORITY_BANK
#endif

#if CONFIG_ADC_ADUC841_HIGHPRIO
#define ADUC841_ADC_BANK		HIGH_PRIORITY_BANK
#else
#define ADUC841_ADC_BANK		LOW_PRIORITY_BANK
#endif

#if CONFIG_ADC_ADUC841_TIMER_HIGHPRIO
#define TIMER1_BANK			HIGH_PRIORITY_BANK
#define TIMER2_BANK			HIGH_PRIORITY_BANK
#else
#define TIMER1_BANK			LOW_PRIORITY_BANK
#define TIMER2_BANK			LOW_PRIORITY_BANK
#endif

#define TIMER1_INTERRUPT		3
#define TIMER2_INTERRUPT		5
#define SERIAL_INTERRUPT		4
#define ADUC841_SPI_IRQ			7

/**
 * irq_enter - enter softirq capable interrupt
 */
#define irq_enter(bank) \
	setup_registers(bank); \
	__asm__(push acc); \
	__asm__(push b); \
	__asm__(push _bp); \
	__asm__(push psw); \
	PSW = bank << 3;

void __irq_exit(void) __using(LOW_PRIORITY_BANK) __naked;
void __highirq_exit(void) __using(HIGH_PRIORITY_BANK) __naked;
#define irq_exit(bank)	\
	if (bank == HIGH_PRIORITY_BANK) { \
		__asm__(ljmp ___highirq_exit); \
	} else { \
		__asm__(ljmp ___irq_exit); \
	}

#define irq_exit_fast(bank) \
	do { \
		__asm__(pop psw); \
		__asm__(pop _bp); \
		__asm__(pop b); \
		__asm__(pop acc); \
		__asm__(reti); \
	} while (0)

#if CONFIG_JIFFIES
void timer_irq(void) __interrupt(1) __using(TIMER_BANK) __naked;
#endif

#if CONFIG_SERIAL_I51_STREAM || CONFIG_RTU_SLAVE_I51 || CONFIG_SLIP_I51 \
	|| CONFIG_SSP_I51

#if CONFIG_SERIAL_I51_HIGHPRIO
#define I51_SERIAL_BANK		HIGH_PRIORITY_BANK
#elif CONFIG_SERIAL_I51_LOWPRIO
#define I51_SERIAL_BANK		LOW_PRIORITY_BANK
#elif CONFIG_SERIAL_I51_POLL
#define I51_SERIAL_BANK		KERNEL_BANK
#define __serial_irq_attr
#else
#error "unknown serial interrupt mode"
#endif

#ifndef __serial_irq_attr
#define __serial_irq_attr \
       	__interrupt(SERIAL_INTERRUPT) __using(I51_SERIAL_BANK)
#endif

void ser_irq(void) __serial_irq_attr;
#endif

#if CONFIG_SERIAL_I51_POLL
#define i51_serial_interrupt_disable() do { } while (0)
#define i51_serial_interrupt_enable() do { } while (0)
#else
#define i51_serial_interrupt_disable() do { ES = 0; } while (0)
#define i51_serial_interrupt_enable() do { ES = 1; } while (0)
#endif

#if CONFIG_RTU_SLAVE_I51
void timer1_irq(void) __interrupt(TIMER1_INTERRUPT) __using(TIMER1_BANK);
#endif

#ifdef CONFIG_ADC_ADUC841

#define ADUC_TIMER_INTERRUPT	TIMER2_INTERRUPT
#define ADUC_TIMER_BANK		TIMER2_BANK
#define ADUC_TIMER2		1

void aduc_timer_irq(void) __interrupt(ADUC_TIMER_INTERRUPT)
		__using(ADUC_TIMER_BANK);
#endif

#endif
