/*
 * tmos - net.h
 *
 * TMOS Networking Support
 * 
 * Copyright (C) 2010  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __TMOS_NET_H
#define __TMOS_NET_H

#include <tmos/config.h>
#include <tmos/types.h>

#if CONFIG_NET
typedef u8 net_size_t;
typedef u8 net_proto_t;

enum {
#if CONFIG_RTU_SLAVE_I51 || CONFIG_SLIP_I51 || CONFIG_SSP_I51
	NET_IFACE_SERIAL0,
#endif
	NET_IFACE_MAX
};

enum {
#ifdef CONFIG_MB_SLAVE_NET
	NET_PROTO_MODBUS,
#else
#define NET_PROTO_MODBUS NET_PROTO_UNDEF
#endif
	NET_PROTO_UNDEF,
	NET_PROTO_MAX,
};

struct net_msg {
	void __kernel *buffer;
	net_size_t size;
#ifdef NET_HEAD
	net_size_t head;
#endif
	net_size_t tail;
#ifdef NET_INTERFACE_IDS
	net_iface_t interface;
#endif
};

#define msg_init(msg)		do { (void) (msg); } while (0)
#define msg_set_size(msg, x)	do { (msg)->size = (x); } while (0)
#define msg_set_buffer(msg, x)	do { (msg)->buffer = (x); } while (0)

#ifdef NET_HEAD
#define msg_set_head(msg, x)	do { (msg)->size = (x); } while (0)
#else
#define msg_set_head(msg, x)	do { (void) (msg); (void) (x); } while (0)
#endif

#define msg_set_tail(msg, x)	do { (msg)->tail = (x); } while (0)

#ifdef NET_INTERFACE_IDS
#define msg_set_interface(msg, x)	do { (msg)->interface = (x); } while (0)
#else
#define msg_set_interface(msg, x) \
	do { (void) (msg); (void) (x); } while (0)
#endif

#define DECLARE_NET_BUFFER(name, size)	uint8_t __kernel name[(size)]

#ifdef CONFIG_NET_REQUEST_TX
extern u8 net_req_tx[(NET_IFACE_MAX + 7) / 8];

#define net_request_tx(iface) \
	do { net_req_tx[(iface) >> 3] |= 1 << ((iface) & 7); } while (0)
#define net_tx_requested(iface) \
	(net_req_tx[(iface) >> 3] & 1 << ((iface) & 7))
#elif CONFIG_NET_REQUEST_TX_NEVER
#define net_request_tx(iface)		do { (void) iface; } while (0)
#define net_tx_requested(iface)		(0)
#else
#define net_request_tx(iface)		do { (void) iface; } while (0)
#define net_tx_requested(iface)		(1)
#endif

net_size_t net_xmit(net_proto_t proto, struct net_msg __kernel *msg)
	__reentrant;

/* network stacks callbacks */
net_size_t modbus_xmit(struct net_msg __kernel *msg) __reentrant;

/* network device drivers callbacks */
void net_softirq(void) __reentrant;

#define NET_IF(name, poll, ioctl) \
	extern void poll(void) __reentrant;
#include <tmos/net-ifs.h>
#undef NET_IF

#define NET_IF(name, poll, ioctl) \
	extern char ioctl(unsigned char cmd, void __kernel *arg) __reentrant;
#include <tmos/net-ifs.h>
#undef NET_IF

#define NET_IOCTL_SET_ADDR_BYTE	1		/* set 1-byte address */

#define NET_IOCTL_SET_SPEED	2
enum {
	NET_SPEED_DEFAULT,
	NET_SPEED_AUTO,
	NET_SPEED_9600,
	NET_SPEED_19200,
	NET_SPEED_38400,
	NET_SPEED_57600,
	NET_SPEED_115200,
	NET_SPEED_125000,
	NET_SPEED_230400,
	NET_SPEED_250000,
	NET_SPEED_460800,
	NET_SPEED_500000,
	NET_SPEED_921600,
	NET_SPEED_1M,
	NET_SPEED_1500000,
	NET_SPEED_10M,
	NET_SPEED_12M,
	NET_SPEED_100M,
};

#ifdef NET_INTERFACE_IDS
char __net_ioctl(unsigned char netif, unsigned char cmd, void __kernel *arg)
	__reentrant;
#define net_ioctl(netif, cmd, arg) __net_ioctl(netif, cmd, arg)
#else
char __net_ioctl(unsigned char cmd, void __kernel *arg) __reentrant;
#define net_ioctl(netif, cmd, arg) __net_ioctl(cmd, arg)
#endif

// #define net_wakeup() raise_softirq_irqoff(NET_SOFTIRQ)

#endif

#define net_init()	do { } while (0)

#endif
